#ifndef __CHAINS__H
#define __CHAINS__H

#include <gtk/gtk.h>

/*
	//
	// ... ipchains rules-chain tree ...
	// 
	// this file manages the tree that contains chains of rules
	// on the display this looks like this:
	//
	//   +--+ chain 1
	//      +-- rule 1
	//      +-- rule 2
	//      +-- rule 3
	//   +--+ chain 2
	//      +-- rule 1
	//      +-- rule 2
	//   +--+ chain 3
	// 
	// each chain has a name and default action (ACCEPT, DENY, REJECT, etc...)
	// as defined in the structure below
	//
	// these functions set the internal variables as well as update the 'rule_tree' widget
	//
*/

/*************************************************************** FUNCTIONS */

/*----------------------------------------- chains only */

/* initialize the chains (the system ones) */
bool chains_init(void);

/* add a new chain with given criteria */
bool chains_add_chain(const char* name, const char* default_action, bool select=false, bool system=false);

/* delete a chain given the name */
bool chains_delete_chain(const GtkLabel* label);

/* rename a chain */
bool chains_rename_chain(const GtkLabel* label, const char* new_name);

/* change default action */
bool chains_change_default_action(const GtkLabel* label, const char* action);

/*----------------------------------------- rules only */

/* add a new rule to the end of the chain */
bool chains_append_rule(const GtkLabel* label, const char* rule_name, const char* rule_def, bool select=false);

/* delete a rule from the chain */
bool chains_delete_rule(const GtkLabel* label, const char* rule_name);

/* rename a rule */
bool chains_rename_rule(const GtkLabel* label, const char* rule_name, const char* new_name);

/* change the rule definition */
bool chains_redefine_rule(const GtkLabel* label, const char* rule_name, const char* rule_def);

/*----------------------------------------- integrated */

/* something was selected */
bool chains_handle_selection(GtkLabel* widget);

/* load from file descriptor */
bool chains_load_from_file(int fd);

/* save to a file descriptor */
bool chains_save_to_file(int fd);

/* perform the file action */
bool do_file_action( char* filemode, char* filename );

#endif
