
#include "processor.h"

#include "global.h"

#include <string.h>
#include <stddef.h>
#include <ctype.h>

// replace with real TCP/UDP/ICMP values as used by IP
#define PROT_TCP 1
#define PROT_UDP 2
#define PROT_ICMP 3


//-------------------------------------------------------------
void default_display(void)
{
	// some stupid defaults...
	gtk_entry_set_text (GTK_ENTRY (rule_type_entry), "User defined rule");	
	// by default we are active...
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_active_on), 1);
	// by default it is not bidirectional...
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_bidirectional), 0);
  // by default it is any protocol...
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_prot_any),1);
	// by default protocol negation is off...
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_prot_not),0);
	// degation of the source and destination IPs is off by default...
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_addr_src_not),0);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_addr_dst_not),0);
	// the src/dst IPs are blank by default
	gtk_entry_set_text (GTK_ENTRY (rule_addr_src_entry), "");
	gtk_entry_set_text (GTK_ENTRY (rule_addr_dst_entry), "");
	// now the port negations to off...
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_tcp_port_src_not),0);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_udp_port_src_not),0);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_tcp_port_dst_not),0);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_udp_port_dst_not),0);
	// blank the port entries...
	gtk_entry_set_text (GTK_ENTRY (rule_tcp_port_src_entry), "");
	gtk_entry_set_text (GTK_ENTRY (rule_udp_port_src_entry), "");
	gtk_entry_set_text (GTK_ENTRY (rule_tcp_port_dst_entry), "");
	gtk_entry_set_text (GTK_ENTRY (rule_udp_port_dst_entry), "");
	// default icmp to not negated and '*' for type
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_icmp_type_not),0);
	gtk_entry_set_text (GTK_ENTRY (rule_icmp_type_entry), "*");
	// default selection of interface
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_int_not),0);
	gtk_entry_set_text (GTK_ENTRY (rule_int_entry), "*");
	// default the jump to part...
	gtk_entry_set_text (GTK_ENTRY (rule_jump_entry), "*");
	gtk_entry_set_text (GTK_ENTRY (rule_redirect_port_entry), "*");
	// default the marking packet value to 0
	gtk_spin_button_set_value( GTK_SPIN_BUTTON(rule_mark_spin), 0 );
	// set logging off
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_log_on),0);
	// initialize the net link
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_netlink_on),0);
	gtk_spin_button_set_value( GTK_SPIN_BUTTON(rule_netlink_spin), 0 );
	// initialize the TOS field
	gtk_entry_set_text( GTK_ENTRY(rule_tos_entry), "Default");
	gtk_label_set_text( GTK_LABEL(rule_tos_note), "(0x00 0x00)" );
	// initialize the fragmenation to off
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_frag_not),0);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_frag_on),0);
	// now initialize the syn matching to off
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_tcp_syn_not),0);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_tcp_syn_on),0);

}

//-------------------------------------------------------------
// helper display functions...

// parse protocol part
bool display_protocol(char** running, const char *delimiters)
{
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_prot_any),0);
	char* token = strsep (running, delimiters);
	if( !token ) return false;
	if( strncmp(token,"!",1)==0 ) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_prot_not),1);
		token = strsep (running, delimiters);
		if( !token ) return false;
	}
	if( strncmp(token,"tcp",3)==0 || atoi(token)==PROT_TCP ){
		gtk_notebook_set_page (GTK_NOTEBOOK(rule_prot_notebook),0);
	}else if( strncmp(token,"udp",3)==0 || atoi(token)==PROT_UDP ){
		gtk_notebook_set_page (GTK_NOTEBOOK(rule_prot_notebook),1);
	}else if( strncmp(token,"icmp",4)==0 || atoi(token)==PROT_ICMP ){
		gtk_notebook_set_page (GTK_NOTEBOOK(rule_prot_notebook),2);
	}else
		return false;
	return true;
}

// parse port part (source or destination)
bool display_port(char type, char** running, const char *delimiters)
{
	char* token = strsep (running, delimiters);
	if( !token ) return false;
	if( strncmp(token,"!",1)==0 ) {
		if( type=='s'){
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_tcp_port_src_not),1);
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_udp_port_src_not),1);			
		}else{
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_tcp_port_dst_not),1);
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_udp_port_dst_not),1);			
		}
		token = strsep (running, delimiters);
		if( !token ) return false;
	}
	if( type=='s'){
		gtk_entry_set_text (GTK_ENTRY (rule_tcp_port_src_entry), token);
		gtk_entry_set_text (GTK_ENTRY (rule_udp_port_src_entry), token);
	}else{
		gtk_entry_set_text (GTK_ENTRY (rule_tcp_port_dst_entry), token);
		gtk_entry_set_text (GTK_ENTRY (rule_udp_port_dst_entry), token);
	}
	return true;
}

// parse address part (source or destination)
bool display_address(char type, char** running, const char *delimiters)
{
	char* token = strsep (running, delimiters);
	if( !token ) return false;
	if( strncmp(token,"!",1)==0 ) {
		if( type=='s') gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_addr_src_not),1);
		else           gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_addr_dst_not),1);
		token = strsep (running, delimiters);
		if( !token ) return false;
	}
	if( type=='s') gtk_entry_set_text (GTK_ENTRY (rule_addr_src_entry), token);
	else           gtk_entry_set_text (GTK_ENTRY (rule_addr_dst_entry), token);
	// do we have the optional port part?
	if( *running==NULL ) return true;
	while( (**running)&&isspace(**running) ) (*running)++;
	if( (**running)&&(**running!='-') )
		return display_port(type,running,delimiters);
	// else we are done...
	return true;
}

// parse icmp-type part
bool display_icmp_type(char** running, const char *delimiters)
{
	char* token = strsep (running, delimiters);
	if( !token ) return false;
	if( strncmp(token,"!",1)==0 ) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_icmp_type_not),1);
		token = strsep (running, delimiters);
		if( !token ) return false;
	}
	// for now we will just trust the user selection of icmp-type
	gtk_entry_set_text (GTK_ENTRY (rule_icmp_type_entry), token);	
	return true;
}

// parse interface part
bool display_interface(char** running, const char *delimiters)
{
	char* token = strsep (running, delimiters);
	if( !token ) return false;
	if( strncmp(token,"!",1)==0 ) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_int_not),1);
		token = strsep (running, delimiters);
		if( !token ) return false;
	}
	// for now we will just trust the user selection of interfaces
	gtk_entry_set_text (GTK_ENTRY (rule_int_entry), token);
	return true;
}

// parse jump part
bool display_jump(char** running, const char *delimiters)
{
	char* token = strsep (running, delimiters);
	if( !token ) return false;
	// for now we will just trust the user selection of jump type
	printf("    setting TOKEN: '%s'\n",token);
	gtk_entry_set_text (GTK_ENTRY (rule_jump_entry), token);
	if( strncmp(token,"REDIRECT",8)==0 ){
		char* token = strsep (running, delimiters);
		if( !token ) return false;
		gtk_entry_set_text (GTK_ENTRY (rule_redirect_port_entry), token);
	}
	return true;
}

// parse mark part
bool display_mark(char** running, const char *delimiters)
{
	char* token = strsep (running, delimiters);
	if( !token ) return false;
	gtk_spin_button_set_value( GTK_SPIN_BUTTON(rule_mark_spin), atoi(token) );
	return true;
}

// parse netlink output part
bool display_netlink_output(char** running, const char *delimiters)
{
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_netlink_on),1);
	// the next one is optional, so we must check if we can get another token
	if( (**running)&&(**running!='-') )
	{
		char* token = strsep (running, delimiters);
		if( !token ) return false;
		gtk_spin_button_set_value( GTK_SPIN_BUTTON(rule_netlink_spin), atoi(token) );
	}
	return true;
}

// parse TOS masks part
bool display_tos_masks(char** running, const char *delimiters)
{
	char* token1 = strsep (running, delimiters);
	if( !token1 ) return false;
	char* token2 = strsep (running, delimiters);
	if( !token2 ) return false;
	char buffer[128];
	snprintf(buffer,128,"(%s %s)",token1,token2);
	gtk_label_set_text( GTK_LABEL(rule_tos_note), buffer );
	return true;
}



//-------------------------------------------------------------
// parse the string and pull out all the valid field that we 
// are aware off... ignore unknow tags
bool rule_to_display(const string& name, const string& str)
{
	int len = str.size();
	if( len==0 ) return false;
	
	const char delimiters[] = " ";
	char rule[len+1];
	char *running,*token;
	bool negateNext=false;

	// copy the rule to a backup storage
	strncpy(rule,str.data(),len);
	rule[str.size()]=0;
	running=rule;
	
	printf("  rule_to_display( '%s', '%s' )\n",name.data(),rule);

	// set all defaults, ie clear the display...
	default_display();
	
	// now set rule name
	gtk_entry_set_text (GTK_ENTRY (rule_name_entry), name.data());	
	
	// go through each token and parse it
	while( (token = strsep (&running, delimiters))!=NULL )
	{
		// first update the token pointer to the beginning of a token
		while((*token)&&isspace(*token)) token++;
		if( !*token) return false;
		
		// now check what token we are dealing with...
		if( strncmp(token,"#",1)==0 ){
			// this one is commented 
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_active_on), 0);
			
		}else if( strncmp(token,"!",1)==0 ){
			negateNext=true;
			
		}else{
			// these entries could possibly be negated
			
			if( strncmp(token,"-b",2)==0 || strncmp(token,"--bidirectional",15)==0 ){
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_bidirectional), 1);
				
			}else if( strncmp(token,"-p",2)==0 || strncmp(token,"--proto",7)==0 ){
				if( !display_protocol( &running, delimiters) ) 
					return false;
				
			}else if( strncmp(token,"-s",2)==0 || strncmp(token,"--source",8)==0 ){
				if( !display_address( 's', &running, delimiters ) )
					return false;
			
			}else if( strncmp(token,"-d",2)==0 || strncmp(token,"--destination",13)==0 ){
				if( !display_address( 'd', &running, delimiters ) )
					return false;
			
			}else if( strncmp(token,"--source-port",12)==0 ){
				if( !display_port( 's', &running, delimiters ) )
					return false;
				
			}else if( strncmp(token,"--destination-port",18)==0 ){
				if( !display_port( 'd', &running, delimiters ) )
					return false;
				
			}else if( strncmp(token,"--icmp-type",11)==0 ){
				if( !display_icmp_type( &running, delimiters ) )
					return false;
				
			}else if( strncmp(token,"-i",2)==0 || strncmp(token,"--interface",11)==0 ){
				if( !display_interface( &running, delimiters ) )
					return false;

			}else if( strncmp(token,"-j",2)==0 || strncmp(token,"--jump",6)==0 ){
				if( !display_jump( &running, delimiters ) )
					return false;
				
			}else if( strncmp(token,"-m",2)==0 || strncmp(token,"--mark",6)==0 ){
				if( !display_mark( &running, delimiters ) )
					return false;

			}else if( strncmp(token,"-l",2)==0 || strncmp(token,"--log",5)==0 ){
				// set the log value on
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_log_on),1);

			}else if( strncmp(token,"-o",2)==0 || strncmp(token,"--output",8)==0 ){
				if( !display_netlink_output( &running, delimiters ) )
					return false;
				
			}else if( strncmp(token,"-t",2)==0 || strncmp(token,"--TOS",5)==0 ){
				if( !display_tos_masks( &running, delimiters ) )
					return false;
				
			}else if( strncmp(token,"-f",2)==0 || strncmp(token,"--fragment",10)==0 ){
				// set the log value on
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_frag_on),1);
				if( negateNext )
					gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_frag_not),1);

			}else if( strncmp(token,"-y",2)==0 || strncmp(token,"--syn",5)==0 ){
				// set the log value on
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_tcp_syn_on),1);
				if( negateNext )
					gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rule_tcp_syn_not),1);

			}else{
				return false;
			}
			
			// turn off the negate flag
			negateNext=false;
		}
	}
	
	// yeay... done...
	return true;
}

//-------------------------------------------------------------
string display_to_rule(void)
{
	string ret;
	
	
	return ret;
}

